/**
    Ein Fuzzysystem umfaßt Eingangsfuzzysets, Ausgangsfuzzysets und Regeln, über die 
    beide miteinander verknüpft werden.
    
    Um den Implementierungsaufwand klein zu halten, werden einige Einschränkungen gemacht,
    was die Struktur des Systems betrifft:
    
    1) Es wird nur ein Ausgangsfuzzyset erlaubt.
    Diese Einschränkung läßt sich leicht durch Einsatz mehrerer FuzzySysteme dieses Typs aufheben.
    
    2) Innerhalb der Regeln wird nur die UND Verknüpfung zwischen je einer Fuzzygröße aus den verschiedenen Fuzzysets zugelassen
    Dabei muß aus jedem Eingangsfuzzyset auch genau eine Fuzzygröße genommen werden.
    Mit dieser Art Regeln läßt sich weitestgehend jede andere Art an Regeln umschreiben.
    
    Eine Regel kann nun durch ein Integer-Array beschrieben werden, 
    in dem Index 0..n-2 jeweils den Index der Fuzzygröße enthält die aus dem i-ten Fuzzyset UND verknüpft werden soll.
    Index n-1 in diesem Array repräsentiert den Index der Ausgangsfuzzygröße des (einen) Ausgangsfuzzysets, welches diese Regel aktiviert.
    
    Beispiel:
    
    2 0 4
    
    Aktivierung von Ausgangsfuzzygröße mit Index 4 = Min(Fuzzygröße 2 im ersten Eingangsfuzzyset, Fuzzygröße 0 im zweiten Eingangsfuzzyset)
    
    Konstruktor:
    übergeben werden die Stützpunkte aller Eingangsfuzzysets und die des Ausgangsfuzzysets und die Regeln als Integer-Array formuliert.
*/

public class FuzzySystem
{
    private Eingangsfuzzyset[] eingangsfuzzyset;
    private Ausgangsfuzzyset ausgangsfuzzyset;
    private int[][] regeln;

    private double[][] eingangs_aktivierungen; //...aller Fuzzygrößen in allen Eingangs-Fuzzysets
    private double[] regelaktivierungen;
  
    public FuzzySystem(double[][] stellen_eingang, double[] stellen_ausgang, int[][] regeln)
    {
        eingangsfuzzyset = new Eingangsfuzzyset[stellen_eingang.length];
        for(int i=0;i<eingangsfuzzyset.length;i++)
        {
            eingangsfuzzyset[i] = new Eingangsfuzzyset(stellen_eingang[i]);
        }
        eingangs_aktivierungen = new double[stellen_eingang.length][];
        ausgangsfuzzyset = new Ausgangsfuzzyset(stellen_ausgang);
        
        //Regeln zur Sicherheit klonen:
        this.regeln = new int[regeln.length][];
        for(int i=0;i<this.regeln.length;i++)
        {
            this.regeln[i] = new int[regeln[i].length];
            for(int k=0;k<this.regeln[i].length;k++)
                this.regeln[i][k] = regeln[i][k];
        }
        
        regelaktivierungen = new double[regeln.length];
    }
    
    public double berechneAusgang(double[] eingang)
    {
         if(eingang==null || eingang.length!=eingangsfuzzyset.length)
         {
              println("ERROR in Fuzzysystem: Anzahl der Eingangssignale undgleich Anzahl Eingangsfuzzysets!");
              return -1.0;
         }
         else
         {
              //Aktivierung aller Fuzzygrößen in allen Eingangsfuzzysets beim aktuellen Eingangssignal bestimmen:
              for(int i=0;i<eingang.length;i++)
              {
                    eingangs_aktivierungen[i] = eingangsfuzzyset[i].berechneAktivierungen(eingang[i]);
              }
              //...daraus die Regelaktivierungen berechnen:
              for(int i=0;i<regeln.length;i++)
              {
                    regelaktivierungen[i] = eingangs_aktivierungen[0][regeln[i][0]];
                    for(int k=1;k<regeln[i].length-1;k++)
                    {
                          if(eingangs_aktivierungen[k][regeln[i][k]]<regelaktivierungen[i])
                               regelaktivierungen[i]=eingangs_aktivierungen[k][regeln[i][k]]; //Minimum suchen (UND-Verkn.)
                    }
              }
              //...daraus wiederum den Ausgangsschwerpunkt bestimmen:
              ausgangsfuzzyset.resetStellwert();
              for(int i=0;i<regeln.length;i++)
              {
                  ausgangsfuzzyset.addAktivierung(regeln[i][regeln[i].length-1],regelaktivierungen[i]);  //ACHTUNG: R1 liefert warm, R2 liefert kalt, darum Indices umgekehrt!!!
              }
              return  ausgangsfuzzyset.holeStellwert();      
         }
    }
}
